@@ -19,10 +19,10 @@ class ViewFits(scriptbase.ScriptBase):
         parser.add_argument('spectrograph', type=str,
                             help='A valid spectrograph identifier: {0}'.format(
                                  ', '.join(available_spectrographs)))
-        parser.add_argument('file', type = str, default = None, help = 'FITS file')
-        parser.add_argument("--list", default=False, action="store_true",
-                            help="List the extensions only?")
-        parser.add_argument("--proc", default=False, action="store_true",
+        parser.add_argument('file', type=str, default=None, help='FITS file')
+        parser.add_argument('--list', default=False, action='store_true',
+                            help='List the extensions only?')
+        parser.add_argument('--proc', default=False, action='store_true',
                             help='Process the image (i.e. orient, overscan subtract, multiply by '
                                  'gain) using pypeit.images.buildimage. Note det=mosaic will not '
                                  'work with this option')
@@ -32,7 +32,7 @@ class ViewFits(scriptbase.ScriptBase):
         parser.add_argument('--det', type=str, default=1,
                             help='Detector number. To mosaic keck_deimos or keck_lris images, '
                                  'set equal to mosaic.')
-        parser.add_argument('--chname', type=str, default='Image', help="Name of Ginga tab")
+        parser.add_argument('--chname', type=str, default='Image', help='Name of Ginga tab')
         return parser
 
     @staticmethod
@@ -68,19 +68,29 @@ class ViewFits(scriptbase.ScriptBase):
             img = hdu[args.exten].data
         else:
             spectrograph = util.load_spectrograph(args.spectrograph)
+            bad_read_message = 'Unable to construct image due to a read or image processing ' \
+                               'error.  Use case interpreted from command-line inputs requires ' \
+                               'a raw image, not an output image product from pypeit.  To show ' \
+                               'a pypeit output image, specify the extension using --exten.  ' \
+                               'Use --list to show the extension names.'
             if args.proc:
-                # Use the arc FramePar since this does not do processing
-                par = spectrograph.default_pypeit_par()['calibrations']['arcframe']
-                par['process']['use_darkimage'] = False
-                par['process']['use_biasimage'] = False
-                par['process']['mask_cr'] = False
-                par['process']['cr_sigrej'] = -1
-
-                img = buildimage.buildimage_fromlist(spectrograph, int(args.det), par,
-                                                     [args.file]).image
+                # Use the biasframe processing parameters because processing
+                # these frames is independent of any other frames (ie., does not
+                # perform bias subtraction or flat-fielding)
+                par = spectrograph.default_pypeit_par()['calibrations']['biasframe']
+                try:
+                    img = buildimage.buildimage_fromlist(spectrograph, int(args.det), par,
+                                                        [args.file]).image
+                except Exception as e:
+                    msgs.error(bad_read_message 
+                               + f'  Original exception -- {type(e).__name__}: {str(e)}')
             else:
                 det = None if args.det == 'mosaic' else int(args.det)
-                img = spectrograph.get_rawimage(args.file, det)[1]
+                try:
+                    img = spectrograph.get_rawimage(args.file, det)[1]
+                except Exception as e:
+                    msgs.error(bad_read_message 
+                               + f'  Original exception -- {type(e).__name__}: {str(e)}')
 
         display.connect_to_ginga(raise_err=True, allow_new=True)
         display.show_image(img,chname=args.chname)
